"use strict";

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var R = require('ramda');
var util_1 = require('./util');
exports.expandImagePath = function (path) {
    if (util_1.isBlank(path)) {
        throw new Error('Image path not specified.');
    }
    var index = R.lastIndexOf('/', path.split(''));
    var basePath = path.substr(0, index + 1);
    var fileName = path.substr(index + 1, path.length);
    var parts = fileName.split('.');
    if (parts.length < 2) {
        throw new Error('An image must have a file extension. [path: ' + path + ']');
    }
    fileName = parts[0];
    var extension = parts[1];
    return {
        basePath: basePath,
        fileName: fileName,
        extension: extension,
        '1x': '' + basePath + fileName + '.' + extension,
        '2x': '' + basePath + fileName + '@2x.' + extension
    };
};
exports.image = function (image1x, image2x) {
    var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : { width: 10, height: 10 };
    var width = options.width;
    var height = options.height;

    var img = global.devicePixelRatio > 1 ? image2x : image1x;
    if (!img) {
        img = image1x;
    }
    if (!img) {
        throw new Error('Must have at least a 1x image.');
    }
    return {
        backgroundImage: 'url(' + img + ')',
        width: width,
        height: height,
        backgroundSize: width + 'px ' + height + 'px',
        backgroundRepeat: 'no-repeat'
    };
};
var mergeAndReplace = function mergeAndReplace(key, value, target) {
    Object.assign(target, value);
    delete target[key];
    return target;
};
var formatImage = function formatImage(key, value, target) {
    var _value = _slicedToArray(value, 4);

    var image1x = _value[0];
    var image2x = _value[1];
    var width = _value[2];
    var height = _value[3];

    if (R.is(Number, image2x)) {
        height = width;
        width = image2x;
        image2x = undefined;
    }
    var options = {
        width: width,
        height: height
    };
    var style = exports.image(image1x, image2x, options);
    mergeAndReplace(key, style, target);
};
exports.toPositionEdges = function (key) {
    var value = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : undefined;

    if (value === undefined || value === null) {
        return undefined;
    }
    if (R.is(String, value) && util_1.isBlank(value)) {
        return undefined;
    }
    if (R.is(Array, value) && value.length === 0) {
        return undefined;
    }
    if (!R.is(Array, value)) {
        value = value.toString().split(' ');
    }
    var edges = value.map(function (item) {
        return util_1.toNumber(item);
    });
    var top = void 0;
    var right = void 0;
    var bottom = void 0;
    var left = void 0;
    var getEdge = function getEdge(index) {
        var edge = edges[index];
        if (edge === null || edge === 'null' || edge === '') {
            return undefined;
        }
        return edge;
    };
    switch (edges.length) {
        case 1:
            top = getEdge(0);
            bottom = getEdge(0);
            left = getEdge(0);
            right = getEdge(0);
            break;
        case 2:
            top = getEdge(0);
            bottom = getEdge(0);
            left = getEdge(1);
            right = getEdge(1);
            break;
        case 3:
            top = getEdge(0);
            left = getEdge(1);
            right = getEdge(1);
            bottom = getEdge(2);
            break;
        default:
            top = getEdge(0);
            right = getEdge(1);
            bottom = getEdge(2);
            left = getEdge(3);
    }
    if (top === undefined && right === undefined && bottom === undefined && left === undefined) {
        return undefined;
    }
    return {
        position: key.toLowerCase(),
        top: top, right: right, bottom: bottom, left: left
    };
};
exports.formatPositionEdges = function (key, target) {
    var styles = exports.toPositionEdges(key, target[key]);
    mergeAndReplace(key, styles, target);
};
var formatAbsoluteCenter = function formatAbsoluteCenter(key, value, target) {
    if (value === true) {
        value = 'xy';
    }
    if (value === false || value === undefined || value === null) {
        return;
    }
    var styles = {
        position: 'absolute',
        left: target.left,
        top: target.top,
        transform: ''
    };
    var stringValue = value.toString().trim().toLowerCase();
    if (stringValue.includes('x')) {
        styles.left = '50%';
    }
    if (stringValue.includes('y')) {
        styles.top = '50%';
    }
    var transform = void 0;
    switch (value) {
        case 'yx':
        case 'xy':
            transform = 'translate(-50%, -50%)';
            break;
        case 'x':
            transform = 'translateX(-50%)';
            break;
        case 'y':
            transform = 'translateY(-50%)';
            break;
        default:
            throw new Error('AbsoluteCenter value \'' + value + '\' not supported.');
    }
    styles.transform = ((target.transform || '') + ' ' + transform).trim();
    mergeAndReplace(key, styles, target);
};
var addVendorPrefix = function addVendorPrefix(key, value, target) {
    var capitalized = '' + key.charAt(0).toUpperCase() + key.slice(1);
    var styles = {};
    styles[key] = value;
    styles['Webkit' + capitalized] = value;
    styles['Moz' + capitalized] = value;
    styles['ms' + capitalized] = value;
    Object.assign(target, styles);
};
exports.css = function () {
    var styles = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    Object.keys(styles).forEach(function (key) {
        var value = styles[key];
        if (R.isNil(value)) {
            delete styles[key];
        } else if (util_1.isPlainObject(value)) {
            styles[key] = exports.css(value);
        } else {
            switch (key) {
                case 'Image':
                    formatImage(key, value, styles);
                    break;
                case 'Absolute':
                    exports.formatPositionEdges(key, styles);
                    break;
                case 'Fixed':
                    exports.formatPositionEdges(key, styles);
                    break;
                case 'AbsoluteCenter':
                    formatAbsoluteCenter(key, value, styles);
                    break;
                case 'userSelect':
                    addVendorPrefix(key, value, styles);
                    break;
                default:
            }
        }
    });
    return styles;
};
exports.css.image = exports.image;
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = exports.css;