"use strict";

var R = require('ramda');
exports.compact = function (value) {
    return R.pipe(R.reject(R.isNil), R.reject(R.isEmpty))(value);
};
exports.isPlainObject = function (value) {
    if (R.is(Object, value) === false) {
        return false;
    }
    var ctr = value.constructor;
    if (typeof ctr !== 'function') {
        return false;
    }
    var prot = ctr.prototype;
    if (R.is(Object, prot) === false) {
        return false;
    }
    if (prot.hasOwnProperty('isPrototypeOf') === false) {
        return false;
    }
    return true;
};
exports.isBlank = function (value) {
    if (value === null || value === undefined) {
        return true;
    }
    if (R.is(Array, value) && exports.compact(value).length === 0) {
        return true;
    }
    if (R.is(String, value) && value.trim() === '') {
        return true;
    }
    return false;
};
exports.isNumeric = function (value) {
    if (exports.isBlank(value)) {
        return false;
    }
    var num = parseFloat(value);
    if (num === undefined) {
        return false;
    }
    if (num.toString().length !== value.toString().length) {
        return false;
    }
    return !Number.isNaN(num);
};
exports.toNumber = function (value) {
    if (exports.isBlank(value)) {
        return value;
    }
    var num = parseFloat(value);
    if (num === undefined) {
        return value;
    }
    if (num.toString().length !== value.toString().length) {
        return value;
    }
    return Number.isNaN(num) ? value : num;
};
exports.toBool = function (value) {
    var defaultValue = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : undefined;

    if (R.isNil(value)) {
        return defaultValue;
    }
    if (R.is(Boolean, value)) {
        return value;
    }
    var asString = value.toString().trim().toLowerCase();
    if (asString === 'true') {
        return true;
    }
    if (asString === 'false') {
        return false;
    }
    return defaultValue;
};
exports.toType = function (value) {
    if (!R.is(String, value)) {
        return value;
    }
    var lowerCase = value.toLowerCase().trim();
    if (lowerCase === 'true') {
        return true;
    }
    if (lowerCase === 'false') {
        return false;
    }
    var num = exports.toNumber(lowerCase);
    if (R.is(Number, num)) {
        return num;
    }
    return value;
};